@@ -11,7 +11,7 @@ from .container import WidgetContainerListContentsMixin, WidgetContainerMixin
 from .divider import Divider
 from .padding import Padding
 from .pile import Pile
-from .widget import Widget, WidgetWrap
+from .widget import Widget, WidgetError, WidgetWrap
 
 if typing.TYPE_CHECKING:
     from collections.abc import Iterable, Sequence
@@ -19,19 +19,19 @@ if typing.TYPE_CHECKING:
     from typing_extensions import Literal
 
 
-class GridFlowError(Exception):
+class GridFlowError(WidgetError):
     pass
 
 
-class GridFlow(WidgetWrap, WidgetContainerMixin, WidgetContainerListContentsMixin):
+class GridFlow(WidgetWrap[Pile], WidgetContainerMixin, WidgetContainerListContentsMixin):
     """
     The GridFlow widget is a flow widget that renders all the widgets it
     contains the same width and it arranges them from left to right and top to
     bottom.
     """
 
-    def sizing(self):
-        return frozenset([Sizing.FLOW])
+    def sizing(self) -> frozenset[Sizing]:
+        return frozenset((Sizing.FLOW, Sizing.FIXED))
 
     def __init__(
         self,
@@ -40,7 +40,8 @@ class GridFlow(WidgetWrap, WidgetContainerMixin, WidgetContainerListContentsMixi
         h_sep: int,
         v_sep: int,
         align: Literal["left", "center", "right"] | Align | tuple[Literal["relative", WHSettings.RELATIVE], int],
-    ):
+        focus: int | Widget | None = None,
+    ) -> None:
         """
         :param cells: iterable of flow widgets to display
         :param cell_width: column width for each cell
@@ -49,8 +50,21 @@ class GridFlow(WidgetWrap, WidgetContainerMixin, WidgetContainerListContentsMixi
             (if more than one row is required to display all the cells)
         :param align: horizontal alignment of cells, one of:
             'left', 'center', 'right', ('relative', percentage 0=left 100=right)
+        :param focus: widget index or widget instance to focus on
         """
-        self._contents = MonitoredFocusList([(w, (WHSettings.GIVEN, cell_width)) for w in cells])
+        prepared_contents: list[tuple[Widget, tuple[Literal[WHSettings.GIVEN], int]]] = []
+        focus_position: int = -1
+
+        for idx, widget in enumerate(cells):
+            prepared_contents.append((widget, (WHSettings.GIVEN, cell_width)))
+            if focus_position < 0 and (focus in {widget, idx} or (focus is None and widget.selectable())):
+                focus_position = idx
+
+        focus_position = max(focus_position, 0)
+
+        self._contents: MonitoredFocusList[tuple[Widget, tuple[Literal[WHSettings.GIVEN], int]]] = MonitoredFocusList(
+            prepared_contents, focus=focus_position
+        )
         self._contents.set_modified_callback(self._invalidate)
         self._contents.set_focus_changed_callback(lambda f: self._invalidate())
         self._contents.set_validate_contents_modified(self._contents_modified)
@@ -58,10 +72,8 @@ class GridFlow(WidgetWrap, WidgetContainerMixin, WidgetContainerListContentsMixi
         self.h_sep = h_sep
         self.v_sep = v_sep
         self.align = align
-        self._cache_maxcol = None
-        super().__init__(None)
-        # set self._w to something other than None
-        self.get_display_widget(((h_sep + cell_width) * len(self._contents),))
+        self._cache_maxcol = self._get_maxcol(())
+        super().__init__(self.generate_display_widget((self._cache_maxcol,)))
 
     def _invalidate(self) -> None:
         self._cache_maxcol = None
@@ -69,7 +81,7 @@ class GridFlow(WidgetWrap, WidgetContainerMixin, WidgetContainerListContentsMixi
 
     def _contents_modified(
         self, slc, new_items: Iterable[tuple[Widget, tuple[Literal["given", WHSettings.GIVEN], int]]]
-    ):
+    ) -> None:
         for item in new_items:
             try:
                 _w, (t, _n) = item
@@ -166,7 +178,7 @@ class GridFlow(WidgetWrap, WidgetContainerMixin, WidgetContainerListContentsMixi
         self.cell_width = width
 
     @property
-    def contents(self):
+    def contents(self) -> MonitoredFocusList[tuple[Widget, tuple[Literal[WHSettings.GIVEN], int]]]:
         """
         The contents of this GridFlow as a list of (widget, options)
         tuples.
@@ -369,27 +381,36 @@ class GridFlow(WidgetWrap, WidgetContainerMixin, WidgetContainerListContentsMixi
             ) from exc
         self.contents.focus = position
 
-    def get_display_widget(self, size: tuple[int]) -> Divider | Pile:
+    def _get_maxcol(self, size: tuple[int] | tuple[()]) -> int:
+        if size:
+            (maxcol,) = size
+        else:
+            maxcol = len(self) * self.cell_width + (len(self) - 1) * self.h_sep
+        return maxcol
+
+    def get_display_widget(self, size: tuple[int] | tuple[()]) -> Divider | Pile:
         """
         Arrange the cells into columns (and possibly a pile) for
         display, input or to calculate rows, and update the display
         widget.
         """
-        (maxcol,) = size
+        maxcol = self._get_maxcol(size)
+
         # use cache if possible
         if self._cache_maxcol == maxcol:
             return self._w
 
         self._cache_maxcol = maxcol
-        self._w = self.generate_display_widget(size)
+        self._w = self.generate_display_widget((maxcol,))
 
         return self._w
 
-    def generate_display_widget(self, size: tuple[int]) -> Divider | Pile:
+    def generate_display_widget(self, size: tuple[int] | tuple[()]) -> Divider | Pile:
         """
         Actually generate display widget (ignoring cache)
         """
-        (maxcol,) = size
+        maxcol = self._get_maxcol(size)
+
         divider = Divider()
         if not self.contents:
             return divider
@@ -433,7 +454,7 @@ class GridFlow(WidgetWrap, WidgetContainerMixin, WidgetContainerListContentsMixi
             del p.contents[:1]
         else:
             # Ensure p __selectable is updated
-            p._contents_modified()
+            p._contents_modified()  # pylint: disable=protected-access
 
         return p
 
@@ -472,33 +493,47 @@ class GridFlow(WidgetWrap, WidgetContainerMixin, WidgetContainerListContentsMixi
             self._set_focus_from_display_widget()
         return key
 
+    def pack(self, size: tuple[int] | tuple[()] = (), focus: bool = False) -> tuple[int, int]:
+        if size:
+            return super().pack(size, focus)
+        cols = len(self) * self.cell_width + (len(self) - 1) * self.h_sep
+        return cols, self.rows((cols,), focus)
+
     def rows(self, size: tuple[int], focus: bool = False) -> int:
         self.get_display_widget(size)
         return super().rows(size, focus=focus)
 
-    def render(self, size: tuple[int], focus: bool = False):
+    def render(self, size: tuple[int] | tuple[()], focus: bool = False):
         self.get_display_widget(size)
         return super().render(size, focus)
 
-    def get_cursor_coords(self, size: tuple[int]) -> tuple[int, int]:
+    def get_cursor_coords(self, size: tuple[int] | tuple[()]) -> tuple[int, int]:
         """Get cursor from display widget."""
         self.get_display_widget(size)
         return super().get_cursor_coords(size)
 
-    def move_cursor_to_coords(self, size: tuple[int], col: int, row: int):
+    def move_cursor_to_coords(self, size: tuple[int] | tuple[()], col: int, row: int):
         """Set the widget in focus based on the col + row."""
         self.get_display_widget(size)
         rval = super().move_cursor_to_coords(size, col, row)
         self._set_focus_from_display_widget()
         return rval
 
-    def mouse_event(self, size: tuple[int], event, button: int, col: int, row: int, focus: bool) -> Literal[True]:
+    def mouse_event(
+        self,
+        size: tuple[int] | tuple[()],
+        event: str,
+        button: int,
+        col: int,
+        row: int,
+        focus: bool,
+    ) -> Literal[True]:
         self.get_display_widget(size)
         super().mouse_event(size, event, button, col, row, focus)
         self._set_focus_from_display_widget()
         return True  # at a minimum we adjusted our focus
 
-    def get_pref_col(self, size: tuple[int]):
+    def get_pref_col(self, size: tuple[int] | tuple[()]):
         """Return pref col from display widget."""
         self.get_display_widget(size)
         return super().get_pref_col(size)
